<?php

/*
 * Error codes when calling 
 * 0 user is authenticated you can check the 
 * 1 user is not authenticated and it was ask not to authenticate.
 * 2 Couldn't get the challenge for the app
 * 3 Wrong challenge
 * 4 Failed to validate the user
 */

function _seagate_sdk_make_call($URL, $Opts, &$Content) {
	$Context = stream_context_create($Opts);
	$Handle = fopen($URL, 'rb', false, $Context);
	if (!$Handle) {
		return 2;
	}
	$Meta = @stream_get_meta_data($Handle);
	$status;
	$auth_error = 0;
	for ($i = 0; $i < count($Meta["wrapper_data"]); $i++) {
		/*
		 * We need to iterate on all the statuses, because
		 * if the page was redirected real status is not index 0
		 */
		$elem = $Meta["wrapper_data"][$i];
		preg_match('|HTTP/\d\.\d\s+(\d+)\s+.*|', $elem, $match);
		if ($match) {
			$status = $match[1];
		}
		if (0 == substr_compare($elem, "auth_error", 0, 10, true)) {
			$temp = explode(":", $elem);
			$auth_error = intval(trim($temp[1]));
		}
	}
	echo $status . "==" . $auth_error;
	if ($status != "200") {
		if (0 != $auth_error) {
			/*
			 * 3 not enough rights to make this call
			 * 4 unknown user token unset the seagate_sdk_user_token and reauthenticate
			 * 5 elapsed user token unset the seagate_sdk_user_token and reauthenticate
			 * 6 unknown application token
			 * 7 elapsed application token
			 */
			return $auth_error;
		}
		return 2;
	} else {
		$Content = @stream_get_contents($Handle); //, $Meta["unread_bytes"]);
	}
	fclose($Handle);
	return 0;
}

function seagate_sdk_get_config() {
	/*
	 * All the information are stored in /etc/unicorn_api.conf if the
	 * application registration was done with the helpers.
	 */
	$unicorn_api_config = file_get_contents("/etc/unicorn_api.conf");
	$config = json_decode($unicorn_api_config, true);
	return $config;
}

function seagate_sdk_open_app_session() {
	$config = seagate_sdk_get_config();
	/*
	 * We need to open a new session for this application.
	 */

	/*
	 * Get the challenge
	 */
	$URL = "http://" . $config["nas_ip"] . "/api/external/authenticate/app_login";
	$Opts = array(
			"http" => array(
					"ignore_errors" => 1,
					"method" => "GET"
			)
	);
	$error = _seagate_sdk_make_call($URL, $Opts, $Content);
	if ($error) {
		/*
		 * Wailed to get challenge...
		 */
		return 2;
	}
	$challenge = json_decode($Content);
	/*
	 * Now that we have the challenge we call to get our token
	 */
	$request = array(
			"app_id" => $config["install_id"],
			"challenge_id" => $challenge->challenge_id,
			"secret" => sha1($challenge->challenge . $config["token"])
	);
	$request_str = json_encode($request, true);
	$Opts["http"]["method"] = "POST";
	$Opts["http"]["content"] = "$request_str";
	$content_length = strlen($request_str);
	$Opts["http"]["header"] = "Content-Type: text/json\r\n" .
					"Content-length: $content_length\r\n";
	$URL = "http://" . $config["nas_ip"] . "/api/external/authenticate/app_login";
	$error = _seagate_sdk_make_call($URL, $Opts, $Content);
	if ($error) {
		/*
		 * Wrong challenge...
		 */
		return 3;
	}
	$resp_json = json_decode($Content);
	$_SESSION["seagate_sdk_app_session"] = $resp_json->session_token;
	/*
	 * Now we can make some calls as an application
	 * In order to do this add a header Authentication: APP_AUTH + $_SESSION["seagate_sdk_app_session"]
	 * to all your calls
	 */
}

function seagate_sdk_get_current_user(&$user, $authenticate, $_retry = false) {
	if (array_key_exists("seagate_sdk_user", $_SESSION)) {
		/* */
		$user = $_SESSION['seagate_sdk_user'];
		return 0;
	} else {
		if (!array_key_exists('seagate_sdk_app_session', $_SESSION)) {
			if (!$authenticate) {
				/*
				 * It was requested not to authenticate automatically.
				 * Just return false in that case.
				 */
				return 1;
			}
			if ($error = seagate_sdk_open_app_session()) {
				return $error;
			}
		}

		/*
		 * Check if the headers are set.
		 */
		if (isset($_GET['app_user_token'])) {
			$_SESSION["seagate_sdk_user_token"] = $_GET["app_user_token"];
			/*
			 * Now we can make some calls on behalf of the current logged in user
			 * Add the APP_USER header + the application authentication header
			 */
			$Opts = array(
					"http" => array(
							"ignore_errors" => 1,
							"method" => "POST"
					)
			);
			$config = seagate_sdk_get_config();
			$Body = "{}";
			$Opts["http"]["header"] = "Content-Type: text/json\r\n" .
							"Content-length: " . strlen($Body) . "\r\n" .
							"Authentication: APP_AUTH " . $_SESSION["seagate_sdk_app_session"] . "\r\n" .
							"APP_USER: " . $_SESSION["seagate_sdk_user_token"] . "\r\n";
			$URL = "http://" . $config["nas_ip"] . "/api/external/5.0/nas_authentication.NasAuth.mySelf";
			$Opts["http"]["content"] = "$Body";
			$error = _seagate_sdk_make_call($URL, $Opts, $Content);
			if ($error) {
				/*
				 * Couldn't validate the authentication of the user
				 */
				if ((6 == $error || 7 == $error) && (!$_retry)) {

					unset($_SESSION["seagate_sdk_app_session"]);
					//If we are here we wanted to authenticate
					return seagate_sdk_get_current_user($user, true, true);
				}
				return 4;
			}
			$json_decoded = json_decode($Content);
			$_SESSION["seagate_sdk_user"] = $json_decoded->myself->__properties__;

			die("<html><head><script>function redir(){window.location='" . $config["path"] . "';}</script></head><body onload='redir()'></body></html>");
		}

		if (!$authenticate) {
			return 1;
		}

		$config = seagate_sdk_get_config();
		$path = $config["path"];
		$redirect_url = "";
		if ($config["nas_ip"] != $config["ip"]) {
			$path = "http://" . $config["ip"] . $path;
			$redirect_url .= "http://" . $config["nas_ip"];
		}
		$url = $redirect_url . "/?app_session_token=" . $_SESSION["seagate_sdk_app_session"] . "&app_path=" . urlencode($path);
		die("<html><script>function redir(){window.location='$url';}</script><body onload='redir()'></body></html>");
	}
}

?>
